require 'net/http'
require 'net/https'
require 'uri'
require 'json'

module Testrail

  STATUS = {
      passed: 1,
      failed: 5,
      skipped: 7,
      pending: 6
  }

  class Client

    def initialize(args)
      @projects = nil
      @sections = Hash.new
      @suite   = Hash.new
      @suites   = []

      @client = APIClient.new args[:url]
      %w(user password project_id).each do |key|
        raise Exception.new("TestRail configuration key :#{key} not set. Cannot continue without it.") if args[key.intern].nil?
        @client.send "#{key}=", args[key.intern] if %w(user password).include? key
      end
    end

    def add_run(project_id, suite_id, run_name)
      @client.send_post("add_run/#{project_id}", { suite_id: suite_id,
                                                   name: run_name })
    end

    #cases
    def get_case(case_id)
      @client.send_get("get_case/#{case_id}")
    end

    #results
    def add_result_for_case(run_id, case_id, result)
      @client.send_post("add_result_for_case/#{run_id}/#{case_id}", result)
    end

    #projects
    def get_project(project_id)
      @client.send_get("get_project/#{project_id}")
    end

  end

  class APIClient
    @url = ''
    @user = ''
    @password = ''

    attr_accessor :user
    attr_accessor :password

    def initialize(base_url)
      if !base_url.match(/\/$/)
        base_url += '/'
      end
      @url = base_url + 'index.php?/api/v2/'
    end

    def send_get(uri)
      _send_request('GET', uri, nil)
    end

    def send_post(uri, data)
      _send_request('POST', uri, data)
    end

    private
    def _send_request(method, uri, data)
      url = URI.parse(@url + uri)
      if method == 'POST'
        request = Net::HTTP::Post.new(url.path + '?' + url.query)
        request.body = JSON.dump(data)
      else
        request = Net::HTTP::Get.new(url.path + '?' + url.query)
      end
      request.basic_auth(@user, @password)
      request.add_field('Content-Type', 'application/json')

      conn = Net::HTTP.new(url.host, url.port)
      if url.scheme == 'https'
        conn.use_ssl = true
        conn.verify_mode = OpenSSL::SSL::VERIFY_NONE
      end
      response = conn.request(request)

      if response.body && !response.body.empty?
        begin
          result = JSON.parse(response.body)
        rescue JSON::ParserError => e
          raise APIError.new "TestRail API request (#{request.method} #{url}) failed\n#{e.class}: #{e.message}"
        end
      else
        result = {}
      end

      if response.code != '200'
        if result && result.key?('error')
          error = '"' + result['error'] + '"'
        else
          error = 'No additional error message received'
        end
        raise APIError.new "TestRail API returned HTTP #{response.code} (#{error})"
      end

      result
    end
  end

  class APIError < StandardError
  end
end